package gov.cms.grouper.snf;

import com.mmm.his.cer.foundation.ComponentVersion;
import java.util.function.Function;

// Latest version should always be listed last
@SuppressWarnings("FieldNamingConvention")
public enum SnfComponentVersion implements ComponentVersion {
  r2(100, 104, (dataVersion) -> new gov.cms.grouper.snf.r2.SnfComponent(dataVersion)),
  r3(105, 210, (dataVersion) -> new gov.cms.grouper.snf.r3.SnfComponent(dataVersion)),
  ;

  private final int from;
  private final int to;
  private final Function<Integer, SnfComponentAbstract> createComponent;

  SnfComponentVersion(int from, int to, Function<Integer, SnfComponentAbstract> createComponent) {
    this.from = from;
    this.to = to;
    this.createComponent = createComponent;
  }

  public static SnfComponentVersion of(int dataVersion) {
    SnfComponentVersion result = null;

    for (SnfComponentVersion rel : SnfComponentVersion.values()) {
      if (rel.from <= dataVersion && dataVersion <= rel.to) {
        result = rel;
        break;
      }
    }

    return result;
  }

  @Override
  public String getPackageValue() { return name(); }

  public int getFrom() {
    return from;
  }

  public int getTo() {
    return to;
  }

  public Function<Integer, SnfComponentAbstract> getCreateComponent() {
    return createComponent;
  }

}
